@@ -1,9 +1,3 @@
-/* Some systems (e.g., SunOS) have header files that erroneously declare
- * inet_addr(), inet_ntoa() and gethostbyname() as taking no arguments.
- * This confuses C++.  To overcome this, we use our own routines,
- * implemented in C.
- */
-
 #ifndef _NET_COMMON_H
 #include "NetCommon.h"
 #endif
@@ -14,38 +8,16 @@
 #include <inetLib.h>
 #endif
 
+/* Some systems (e.g., SunOS) have header files that erroneously declare inet_addr() as taking no arguments.
+ * This confuses C++.  To overcome this, we use our own routine, implemented in C.
+ */
+
 unsigned our_inet_addr(cp)
 	char const* cp;
 {
 	return inet_addr(cp);
 }
 
-char *
-our_inet_ntoa(in)
-        struct in_addr in;
-{
-#ifndef VXWORKS
-  return inet_ntoa(in);
-#else
-  /* according the man pages of inet_ntoa :
-
-     NOTES
-     The return value from inet_ntoa() points to a  buffer  which
-     is  overwritten on each call.  This buffer is implemented as
-     thread-specific data in multithreaded applications.
-
-     the vxworks version of inet_ntoa allocates a buffer for each
-     ip address string, and does not reuse the same buffer.
-
-     this is merely to simulate the same behaviour (not multithread
-     safe though):
-  */
-  static char result[INET_ADDR_LEN];
-  inet_ntoa_b(in, result);
-  return(result);
-#endif
-}
-
 #if defined(__WIN32__) || defined(_WIN32)
 #ifndef IMN_PIM
 #define WS_VERSION_CHOICE1 0x202/*MAKEWORD(2,2)*/
@@ -83,16 +55,6 @@ int initializeWinsockIfNecessary(void) { return 1; }
 #define NULL 0
 #endif
 
-#if !defined(VXWORKS)
-struct hostent* our_gethostbyname(name)
-     char* name;
-{
-	if (!initializeWinsockIfNecessary()) return NULL;
-
-	return (struct hostent*) gethostbyname(name);
-}
-#endif
-
 #ifdef USE_SYSTEM_RANDOM
 /* Use the system-supplied "random()" and "srandom()" functions */
 #include <stdlib.h>
@@ -459,10 +421,11 @@ long our_random() {
 #endif
 
 u_int32_t our_random32() {
-  // Return a 32-bit random number.
-  // Because "our_random()" returns a 31-bit random number, we call it a second
-  // time, to generate the high bit.
-  // (Actually, to increase the likelhood of randomness, we take the middle 16 bits of two successive calls to "our_random()")
+  /* Return a 32-bit random number.
+     Because "our_random()" returns a 31-bit random number, we call it a second
+     time, to generate the high bit.
+     (Actually, to increase the likelihood of randomness, we take the middle 16 bits of two successive calls to "our_random()")
+  */
   long random_1 = our_random();
   u_int32_t random16_1 = (u_int32_t)(random_1&0x00FFFF00);
 
