@@ -637,7 +637,7 @@ RTPSink* MatroskaFile
       delete[] identificationHeader; delete[] commentHeader; delete[] setupHeader;
     } else if (strcmp(track->mimeType, "video/RAW") == 0) {
       result = RawVideoRTPSink::createNew(envir(), rtpGroupsock, rtpPayloadTypeIfDynamic, 
-                                          track->pixelHeight, track->pixelWidth, track->bitDepth, track->colorSampling, track->colorimetry);
+                                          track->pixelWidth, track->pixelHeight, track->bitDepth, track->colorSampling, track->colorimetry);
     } else if (strcmp(track->mimeType, "video/H264") == 0) {
       u_int8_t* sps; unsigned spsSize;
       u_int8_t* pps; unsigned ppsSize;
@@ -908,6 +908,10 @@ void MatroskaDemux::seekToTime(double& seekNPT) {
   if (fOurParser != NULL) fOurParser->seekToTime(seekNPT);
 }
 
+void MatroskaDemux::pause() {
+  if (fOurParser != NULL) fOurParser->pause();
+}
+
 void MatroskaDemux::handleEndOfFile(void* clientData) {
   ((MatroskaDemux*)clientData)->handleEndOfFile();
 }
@@ -937,6 +941,18 @@ void MatroskaDemux::handleEndOfFile() {
   delete[] tracks;
 }
 
+void MatroskaDemux::resetState() {
+  // Iterate through all of our 'demuxed tracks', calling 'reset()' on each one.
+  HashTable::Iterator* iter = HashTable::Iterator::create(*fDemuxedTracksTable);
+  MatroskaDemuxedTrack* demuxedTrack;
+  char const* trackNumber;
+
+  while ((demuxedTrack = (MatroskaDemuxedTrack*)iter->next(trackNumber)) != NULL) {
+    demuxedTrack->reset();
+  }
+  delete iter;
+}
+
 
 ////////// CuePoint implementation //////////
 
